// AboutDialog.cpp : implementation file
//

#include "stdafx.h"
#include "FccDemo.h"
#include "AboutDialog.h"


// CAboutDialog dialog

IMPLEMENT_DYNAMIC(CAboutDialog, CDialog)

CAboutDialog::CAboutDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CAboutDialog::IDD, pParent)
	, m_compCount(0)
{
}

CAboutDialog::~CAboutDialog()
{
}

void CAboutDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_COMP_LIST, m_comp);
	DDX_Control(pDX, IDC_BUTTON_REFRESH, m_button);
}


BEGIN_MESSAGE_MAP(CAboutDialog, CDialog)
	ON_WM_SIZE()
END_MESSAGE_MAP()

// CAboutDialog message handlers

void CAboutDialog::OnSize(UINT /*nType*/, int cx, int cy)
{
	// resize log to take all available space
	RECT rectDialog = {0, 0, cx, cy};
	ClientToScreen(&rectDialog);

	RECT rectList, rectButton;
	m_comp.GetWindowRect(&rectList);
	m_button.GetWindowRect(&rectButton);
	rectList.left = 3;
	rectList.top = rectButton.bottom + 3;
	rectList.right = rectDialog.right - 3;
	rectList.bottom = rectDialog.bottom - 3;
	ScreenToClient(&rectList);
	m_comp.MoveWindow(&rectList);
}

void CAboutDialog::ClearComponents()
{
	m_compCount = 0;
}

void CAboutDialog::AddComponent(CString name, CString version, CString isRegistered, CString copyright)
{
	m_title[m_compCount] = name;
	m_version[m_compCount] = version;
	m_registered[m_compCount] = isRegistered;
	m_copyright[m_compCount] = copyright;
	++m_compCount;
}

void CAboutDialog::AddComponent(NLibraryInfo *li, CString isRegistered)
{
	char buffer[100];

	sprintf(buffer, "%d.%d.%d.%d",
		li->VersionMajor, li->VersionMinor, li->VersionBuild, li->VersionRevision);

	m_title[m_compCount] = li->Title;
	m_version[m_compCount] = buffer;
	m_registered[m_compCount] = isRegistered;
	m_copyright[m_compCount] = li->Copyright;
	++m_compCount;
}

void CAboutDialog::AddComponent(wchar_t *filename)
{
	DWORD dummy, size;

	wchar_t strPath[1024];
	size = GetModuleFileName(NULL, strPath, 1024);
	if (size <= 0)
	{
		return;
	}
	for (int i = wcslen(strPath) - 1; (i >= 0) && (strPath[i] != '\\'); --i)
	{
		strPath[i] = 0;
	}

	wchar_t strFilename[1024];
	wsprintf(strFilename, _T("%s%s"), strPath, filename);
	size = GetFileVersionInfoSize(strFilename, &dummy);
	if (size <= 0)
	{
		return;
	}

	void *info = malloc(size);
	if (!GetFileVersionInfo(strFilename, dummy, size, info))
	{
		return;
	}

	struct LANGANDCODEPAGE {
		WORD wLanguage;
		WORD wCodePage;
	} *lpTranslate;

	// Read the list of languages and code pages.
	UINT cbTranslate = 0;

	VerQueryValue(info, 
		TEXT("\\VarFileInfo\\Translation"),
		(LPVOID*)&lpTranslate,
		&cbTranslate);

	// Read the file description for each language and code page.

	wchar_t subBlock[1024];
	void *buffer;
	UINT dwBytes;
	for (UINT i = 0; i < (cbTranslate / sizeof(struct LANGANDCODEPAGE)); ++i)
	{
		wsprintf(subBlock,
			TEXT("\\StringFileInfo\\%04x%04x\\FileDescription"),
			lpTranslate[i].wLanguage, lpTranslate[i].wCodePage);
		VerQueryValue(info, subBlock, &buffer, &dwBytes);
		CString name = (wchar_t *) buffer;

		wsprintf(subBlock,
			TEXT("\\StringFileInfo\\%04x%04x\\FileVersion"),
			lpTranslate[i].wLanguage, lpTranslate[i].wCodePage);
		VerQueryValue(info, subBlock, &buffer, &dwBytes);
		CString version = (wchar_t *) buffer;
		version.Replace(_T(", "), _T("."));

		wsprintf(subBlock,
			TEXT("\\StringFileInfo\\%04x%04x\\LegalCopyright"),
			lpTranslate[i].wLanguage, lpTranslate[i].wCodePage);
		VerQueryValue(info, subBlock, &buffer, &dwBytes);
		CString copyright = (wchar_t *) buffer;

		AddComponent(name, version, _T(""), copyright);

		if (i == 0)
		{
			break;
		}
	}

	free(info);
}

BOOL CAboutDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

	int index;

	index = m_comp.InsertColumn(0, _T("Component"), LVCFMT_LEFT);
	index = m_comp.InsertColumn(1, _T("Version"), LVCFMT_LEFT);
	index = m_comp.InsertColumn(2, _T("Is Registered"), LVCFMT_LEFT);
	index = m_comp.InsertColumn(3, _T("Copyright"), LVCFMT_LEFT);

	for (int i = 0; i < m_compCount; ++i)
	{
		index = m_comp.InsertItem(i, m_title[i]);
		m_comp.SetItemText(index, 1, m_version[i]);
		m_comp.SetItemText(index, 2, m_registered[i]);
		m_comp.SetItemText(index, 3, m_copyright[i]);
	}

	m_comp.SetColumnWidth(0, LVSCW_AUTOSIZE);
	m_comp.SetColumnWidth(1, LVSCW_AUTOSIZE);
	m_comp.SetColumnWidth(2, LVSCW_AUTOSIZE);
	m_comp.SetColumnWidth(3, LVSCW_AUTOSIZE);

	return TRUE;
}
