// FccDemoDlg.h : header file
//

#pragma once

#include "afxwin.h"

#include "EnrollDialog.h"
#include "IdentifyDialog.h"
#include "OptionsDialog.h"
#include "AboutDialog.h"

const wchar_t fccDatabase[] = _T("facedatabase.fcc");
const int maxIdLength = 65;

typedef struct FaceRecord_
{
	wchar_t Id[maxIdLength];
	NSizeType Size;
	void *Template;
} FaceRecord;

// CFccDemoDlg dialog
class CFccDemoDlg : public CDialog
{
// Construction
public:
	CFccDemoDlg(CWnd* pParent = NULL);	// standard constructor

// Dialog Data
	enum { IDD = IDD_FCCDEMO_DIALOG };

#ifdef WIN32_PLATFORM_WFSP
protected:  // control bar embedded members
	CCommandBar m_dlgCommandBar;
#endif // WIN32_PLATFORM_WFSP

protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support

// Implementation
protected:
	HICON m_hIcon;

private:
	void EnumerateComponents(bool log);

	bool m_cameraSupported;

	HNImage m_image;
	HNImage m_grayscale;
	FcceDetectionDetails m_details;
	void ClearImages();
	void SetImage(HNImage image, HNImage grayscale);

private:
	HFccExtractor m_extractor;
	HFccMatcher m_matcher;
	NSizeType m_bufferSize;

	int m_recordCount;
	FaceRecord *m_records;
	void ClearRecords();
	void LoadRecords();
	void SaveRecords();

	HNImage GetImage(int from);
	void *GetTemplate(int from, const wchar_t *id, NSizeType *size);

public:
	enum { FCCDEMO_FROM_FILE = 0, FCCDEMO_FROM_CAMERA };
	enum { FCCDEMO_SINGLE_IMAGE = 0, FCCDEMO_IMAGE_SEQ, FCCDEMO_GENERALIZATION };

	CEdit m_log;
	CButton m_button;
	CEnrollDialog m_enrollDialog;
	CIdentifyDialog m_identifyDialog;
	COptionsDialog m_optionsDialog;
	CAboutDialog m_aboutDialog;

	// Generated message map functions
	virtual BOOL OnInitDialog();
#if defined(_DEVICE_RESOLUTION_AWARE) && !defined(WIN32_PLATFORM_WFSP)
	afx_msg void OnSize(UINT /*nType*/, int /*cx*/, int /*cy*/);
#endif
	DECLARE_MESSAGE_MAP()
public:
	afx_msg void OnBnClickedButtonExtract();
	afx_msg void OnBnClickedButtonIdentify();
	afx_msg void OnBnClickedButtonOptions();
	afx_msg void OnBnClickedButtonAbout();
	afx_msg void OnBnClickedButtonClearDB();
	afx_msg void OnClose();
	afx_msg void OnPaint();
};
